'
'*************************************************************************************************************
'* This source code is provided 'as is' and any express or implied warranties, including,                    *
'* but not limited to, the implied warranties of merchantability and fitness for a particular                *
'* purpose are disclaimed.                                                                                   *
'*                                                                                                           *
'* In no event shall the author, Crownhill or contributers be liable for any direct, indirect, incidental,   *
'* special, exemplary, or consequential damages (including, but not limited to,  procurement of substitute   *
'* goods or services; loss of use, data, or profits;  or business interruption) however caused and on any    *
'* theory of liability, whether in contract, strict liability, or tort (including negligence or otherwise)   *
'* arising in any way out of the use of this source code, even if advised of the possibility of such damage. *
'*                                                                                                           *
'* Any views and conclusions contained in the source code and any documentation are those of the author's    *
'* and should not be interpreted as representing official policies,                                          *
'* either expressed or implied, of Crownhill Associates Ltd.                                                 *
'*************************************************************************************************************
'
' Demonstrate input operation on a Microchip MCP23S17 16-bit SPI I/O expander
'
' The demo will track the changes on PORTA and output it on PORTB
'
' Written for the Crownhill Proton compiler version 3.5.3.1 onwards using an 18F device
'
' Author A. Freixanet (23/02/2016) based on Les Johnson' code 23-02-2012
'
' You are free to use this code as you see fit, however, a mention of the author and compiler would be appreciated.
' The code is offered as-is and Crownhill nor the author take any responsibility for its use.
'-------------------------------------------------------------------------------
    Include "D:/MyIncFiles/18F45K22.inc"            ' Configure the compiler to use the 18F25K20 at 64MHz for the Amicus18 board
'    Include "18F45K22.inc"

'-------------------------------------------------------------------------------
' Declare the Chip Select pins
    $define SPI_CS0_Pin   LATA.1                    ' MCP23S17 CS line connects here
    $define CS0_Ansel_Pin ANSELA.1

    Include "D:/MyIncFiles/18F4xKxx_HSPI.inc"       ' Load the SPI macros into the program
'    Include "18F4xKxx_HSPI.inc"
'-------------------------------------------------------------------------------
    Symbol MCP_Reset = LATA.4             ' MCP23S17 CS line RESET
    Dim bTemp As Byte                   ' Used to hold the input value
'
'-------------------------------------------------------------------------------
MAIN:
' Main program starts here
'
    High MCP_Reset                      ' Pin reset High ( no reset)
'
' Open the SPI peripheral:
'                       Fosc/16 (4MHz operation)
'                       SPI mode 0:0
'                       Sample at end of the clock cycle
    OpenSPI1(SPI_FOSC_16, MODE_00, SMPEND)
'
' Initialise the MCP23S17
'
' At initialisation, Chip_Address must be forced to "000"
' regardless of actual Mcpchip_Address value.
' After initialisation, once IOCON.HAEN is enabled,
' Mcpchip_Address is taken into account by the chip.
'
'    Clear MCP_CS            ' Enable the SPI interface
    ClrCS0                  ' Enable the SPI interface
    DelayCS 1               ' A one cycle delay before starting
    SPI_Write(%11110001)    ' Force ChipAdd to 000
    SPI_Write($0A)          ' Writing to IOCONA writes also to IOCONB
    SPI_Write(%00011000)    ' Init value for IOCON
'    Set MCP_CS              ' Disable the SPI interface
    SetCS0                   ' Disable the SPI interface
'
' Write to the Tris of PortB and set it to all outputs
'
'    Clear MCP_CS            ' Enable the SPI interface
    ClrCS0                  ' Enable the SPI interface
    DelayCS 1               ' A one cycle delay before starting
    SPI_Write(%01000000)    ' Target address 000 and set the R/W Line To Write (0)
    SPI_Write($01)          ' We want to access the IODIRB register
    SPI_Write(%00000000)    ' Set to all outputs
'    Set MCP_CS              ' Disable the SPI interface
    SetCS0                   ' Disable the SPI interface
'
' Write to the Tris of PortA and set it to all inputs
'
'    Clear MCP_CS            ' Enable the SPI interface
    ClrCS0                  ' Enable the SPI interface
    DelayCS 1               ' A one cycle delay before starting
    SPI_Write(%01000000)    ' Target address 000 and set the R/W Line To Write (0)
    SPI_Write($00)          ' We want to access the IODIRB register
    SPI_Write(%11111111)    ' Set to all inputs
'    Set MCP_CS              ' Disable the SPI interface
    SetCS0                   ' Disable the SPI interface
'
' Enable pull-up resistors on PortA
'
'    Clear MCP_CS            ' Enable the SPI interface
    ClrCS0                  ' Enable the SPI interface
    DelayCS 1               ' A one cycle delay before starting
    SPI_Write(%01000000)    ' Target address 000 and set the R/W Line To Write (0)
    SPI_Write($0C)          ' We want to access the GPPUA register
    SPI_Write(%11111111)    ' Set all outputs high
'    Set MCP_CS              ' Disable the SPI interface
    SetCS0                   ' Disable the SPI interface
'
' Display the contents of PortA inputs on PortB outputs
'
    While 1 = 1                 ' Create an infinite loop
        '
        ' Read the condition of PortA
        '
SPIREAD:
'        Clear MCP_CS            ' Enable the SPI interface
        ClrCS0                  ' Enable the SPI interface
        DelayCS 1               ' A one cycle delay before starting
        SPI_Write(%01000001)   ' Target address 000 and set the R/W line to Read (1)
        SPI_Write($12)         ' We want to access the GPIOA register
        bTemp = SPI_Read()     ' Read from the SPI interface
'        Set MCP_CS              ' Disable the SPI interface
        SetCS0                   ' Disable the SPI interface
        '
        ' Write the contents of bTemp to the Latch of PortB
        DelayUS 10
        '
SPIWRITE:
'        Clear MCP_CS            ' Enable the SPI interface
        ClrCS0                  ' Enable the SPI interface
        DelayCS 1               ' A one cycle delay before starting
        SPI_Write(%01000000)    ' Target address 000 and set the R/W Line To Write (0)
        SPI_Write($15)          ' We want to access the OLATB register
        SPI_Write(~bTemp)       ' Write complement of bTemp to the latches of PortB
'        Set MCP_CS              ' Disable the SPI interface
        SetCS0                   ' Disable the SPI interface

        DelayMS 100
    Wend
